@@ -22,11 +22,14 @@ from __future__ import print_function
 
 import json
 import logging
+import sys
+import warnings
 
 import click
 
 import feed2exec
-from feed2exec.feeds import (FeedManager, FeedConfStorage, FeedCacheStorage, Feed)
+from feed2exec.controller import FeedManager
+from feed2exec.model import (FeedConfStorage, FeedItemCacheStorage, Feed)
 import feed2exec.logging
 import feed2exec.plugins as plugins
 from feed2exec.utils import slug
@@ -35,10 +38,10 @@ from feed2exec.utils import slug
 @click.group(help=feed2exec.__description__,
              context_settings=dict(help_option_names=['-h', '--help']))
 @click.version_option(version=feed2exec.__version__)
-@click.option('--loglevel', 'loglevel',
-              help='choose specific log level [default: WARNING]',
-              type=click.Choice(feed2exec.logging.levels),
-              flag_value='WARNING', default=True)
+@click.option('-q', '--quiet', 'loglevel', flag_value='ERROR',
+              help='hide warnings (loglevel: ERROR)')
+@click.option('-w', '--warnings', 'loglevel', flag_value='WARNING', hidden=True,
+              help='show warnings (loglevel: WARNING), default', default=True)
 @click.option('-v', '--verbose', 'loglevel', flag_value='INFO',
               help='show what is happening (loglevel: VERBOSE)')
 @click.option('-d', '--debug', 'loglevel', flag_value='DEBUG',
@@ -48,15 +51,25 @@ from feed2exec.utils import slug
 @click.option('--config', default=None,
               help='use a different config file [default: %s]' % FeedConfStorage.guess_path())
 @click.option('--database', default=None,
-              help='use a different database [default: %s]' % FeedCacheStorage.guess_path())
+              help='use a different database [default: %s]' % FeedItemCacheStorage.guess_path())
 @click.pass_context
 def main(ctx, loglevel, syslog, config, database):
     feed2exec.logging.advancedConfig(level=loglevel, syslog=syslog,
-                                     logFormat='%(messageq)s')
+                                     logFormat='%(message)s')
     if ctx.obj is None:
         ctx.obj = {}
     if database is None:
-        database = FeedCacheStorage.guess_path()
+        # we do this crazy warning dance because guess_path() triggers
+        # a warning, and does not log on its own. for some reason,
+        # warnings do not show up by default in feed2exec, and I don't
+        # have time to investigate why, so this will have to do for
+        # now.
+        with warnings.catch_warnings(record=True) as w:
+            warnings.simplefilter("always")
+            database = FeedItemCacheStorage.guess_path()
+            if (len(w) > 0):
+                logging.warning(str(w[-1].message))
+                assert issubclass(w[-1].category, DeprecationWarning)
         logging.debug('guessed db_path to be %s', database)
     if config is None:
         config = FeedConfStorage.guess_path()
@@ -64,7 +77,12 @@ def main(ctx, loglevel, syslog, config, database):
     ctx.obj['database'] = database
     ctx.obj['config'] = config
     # preload for commands who do not need a specific pattern like add/list/rm
-    ctx.obj['feeds'] = FeedManager(config, database)
+    if 'feed_manager_override' in ctx.obj:
+        ctx.obj['feed_manager'] = ctx.obj['feed_manager_override']
+        logging.debug('using feed_manager_override (%s)', ctx.obj['feed_manager'])
+    else:
+        ctx.obj['feed_manager'] = FeedManager(config, database)
+    logging.debug('using feed_manager (%s)', ctx.obj['feed_manager'])
 
 
 @click.command(help='add a URL to the configuration')
@@ -83,18 +101,18 @@ def main(ctx, loglevel, syslog, config, database):
 @click.pass_obj
 def add(obj, name, url, output, args, filter, filter_args, folder, mailbox):
     try:
-        obj['feeds'].conf_storage.add(name=name, url=url,
-                                      output=plugins.resolve(output), args=args,
-                                      filter=plugins.resolve(filter), filter_args=filter_args,
-                                      folder=folder, mailbox=mailbox)
+        obj['feed_manager'].conf_storage.add(name=name, url=url,
+                                             output=plugins.resolve(output), args=args,
+                                             filter=plugins.resolve(filter), filter_args=filter_args,
+                                             folder=folder, mailbox=mailbox)
     except AttributeError:
-        raise click.BadParameter('feed %s already exists in %s' % (name, obj['feeds'].conf_storage.path))
+        raise click.BadParameter('feed %s already exists in %s' % (name, obj['feed_manager'].conf_storage.path))
 
 
 @click.command(help='list configured feeds')
 @click.pass_obj
 def ls(obj):
-    for feed in obj['feeds'].conf_storage:
+    for feed in obj['feed_manager'].conf_storage:
         if feed:
             print(json.dumps(feed, indent=2, sort_keys=True))
 
@@ -103,7 +121,7 @@ def ls(obj):
 @click.argument('name')
 @click.pass_obj
 def rm(obj, name):
-    obj['feeds'].conf_storage.remove(name)
+    obj['feed_manager'].conf_storage.remove(name)
 
 
 @click.command(help='fetch and process all feeds')
@@ -116,12 +134,10 @@ def rm(obj, name):
 @click.option('--catchup', '-n',
               is_flag=True, help='tell output plugins to do nothing permanent')
 def fetch(obj, pattern, parallel, jobs, force, catchup):
-    st = FeedManager(obj['config'], obj['database'], pattern=pattern)
-    # used for unit testing
-    if obj and obj.get('session'):
-        Feed._session = obj['session']
+    feed_manager = obj['feed_manager']
+    feed_manager.pattern = pattern
     parallel = jobs or parallel
-    st.fetch(parallel, force=force, catchup=catchup)
+    feed_manager.fetch(parallel, force=force, catchup=catchup)
 
 
 @click.command(help='fetch and parse a single feed')
@@ -141,26 +157,34 @@ def fetch(obj, pattern, parallel, jobs, force, catchup):
 @click.pass_obj
 def parse(obj, url, **kwargs):
     kwargs.update({'url': url})
+    feed_manager = obj['feed_manager']
     for plugin in ('output', 'filter'):
         kwargs[plugin] = plugins.resolve(kwargs[plugin])
     feed = Feed(slug(url), kwargs)
-    data = feed.parse(feed.fetch())
-    # XXX: i don't like this - we should use a clean environment.
-    obj['feeds'].dispatch(feed, data, lock=False, force=True)
+    body = feed_manager.fetch_one(feed)
+    if body is None:
+        logging.info('feed already cached, skipping')
+        return 0
+    data = feed.parse(body)
+    if data:
+        # XXX: i don't like this - we should use a clean environment.
+        feed_manager.dispatch(feed, data, lock=False, force=True)
+    else:
+        sys.exit(1)
 
 
 @click.command(name='import', help='import feed list from OPML file')
 @click.argument('path', type=click.File('rb'))
 @click.pass_obj
 def import_(obj, path):
-    obj['feeds'].opml_import(path)
+    obj['feed_manager'].opml_import(path)
 
 
 @click.command(help='export feeds to an OPML file')
 @click.argument('path', type=click.File('wb'))
 @click.pass_obj
 def export(obj, path):
-    obj['feeds'].opml_export(path)
+    obj['feed_manager'].opml_export(path)
 
 
 main.add_command(add)
